@@ -463,12 +463,13 @@ void RGB_to_YUV420(frame *rgb,
 CAMLprim value caml_rgb_of_YUV420(value yuv, value dst)
 {
   CAMLparam2(yuv,dst);
+  CAMLlocal2(y_val,uv_val);
   frame rgb;
   frame_of_value(dst, &rgb);
-  value y_val = Field(yuv, 0);
+  y_val = Field(yuv, 0);
   unsigned char *y = Caml_ba_data_val(Field(y_val, 0));
   int y_stride = Int_val(Field(y_val, 1));
-  value uv_val = Field(yuv, 1);
+  uv_val = Field(yuv, 1);
   unsigned char *u = Caml_ba_data_val(Field(uv_val, 0));
   unsigned char *v = Caml_ba_data_val(Field(uv_val, 1));
   int uv_stride = Int_val(Field(uv_val, 2));
@@ -481,6 +482,41 @@ CAMLprim value caml_rgb_of_YUV420(value yuv, value dst)
   CAMLreturn(Val_unit);
 }
 
+CAMLprim value caml_yuv_of_string(value yuv, value s)
+{
+  CAMLparam2(yuv, s);
+  CAMLlocal1(tmp);
+
+  tmp = Field(yuv,0);
+  int width = Int_val(Field(yuv,1));
+  int height = Int_val(Field(yuv,2));
+  unsigned char *y = Caml_ba_data_val(Field(Field(tmp,0),0));
+  int y_stride = Int_val(Field(Field(tmp,0),1));
+  tmp = Field(tmp,1);
+  unsigned char *u = Caml_ba_data_val(Field(tmp,0));
+  unsigned char *v = Caml_ba_data_val(Field(tmp,1));
+  int uv_stride = Int_val(Field(tmp,2));
+
+  int datalen = width*height*6/4;
+
+  unsigned char *data = (unsigned char*)memalign(ALIGNMENT_BYTES, datalen);
+  if (data == NULL) caml_raise_out_of_memory();
+  memcpy(data, String_val(s), datalen);
+  int i, j;
+
+  caml_enter_blocking_section();
+  for (i = 0; i < height; i++)
+    memcpy(y+i*y_stride, data+i*width, width);
+  for (i = 0; i < height/2; i++) {
+    memcpy(u+i*uv_stride, data+datalen*4/6+i*width/2, width/2);
+    memcpy(v+i*uv_stride, data+datalen*5/6+i*width/2, width/2);
+  }
+  caml_leave_blocking_section();
+
+  free(data);
+  CAMLreturn(Val_unit);
+}
+
 CAMLprim value caml_yuv_blank(value f)
 {
   CAMLparam1(f);
@@ -495,9 +531,10 @@ CAMLprim value caml_yuv_blank(value f)
 CAMLprim value caml_rgb_to_YUV420(value f, value yuv)
 {
   CAMLparam2(f,yuv);
+  CAMLlocal1(tmp);
   frame rgb;
   frame_of_value(f, &rgb);
-  value tmp = Field(yuv, 0);
+  tmp = Field(yuv, 0);
   unsigned char *y = Caml_ba_data_val(Field(tmp,0));
   tmp = Field(yuv,1);
   unsigned char *u = Caml_ba_data_val(Field(tmp,0));
@@ -555,6 +592,8 @@ CAMLprim value caml_rgb_of_rgb8_string(value _rgb, value _data)
     }
   caml_leave_blocking_section();
 
+  free(data);
+
   CAMLreturn(Val_unit);
 }
 
@@ -1003,7 +1042,7 @@ CAMLprim value caml_rgba_swap_rb(value _rgba)
   CAMLreturn(Val_unit);
 }
 
-CAMLprim value caml_rgb_add(value _dst, value _src)
+CAMLprim value caml_rgb_add(value _src, value _dst)
 {
   CAMLparam2(_src,_dst);
   frame src,dst;
